<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/prixmasse/class/prixmasse.class.php
 *  \ingroup    prixmasse
 *  \brief      File of class to manage predefined products sets
 */
require_once DOL_DOCUMENT_ROOT.'/core/class/commonobject.class.php';
require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';

dol_include_once("/prixmasse/class/prixmasse.produit.class.php");

/**
 * Class to manage products or services
 */
class PrixMasse extends CommonObject
{
	public $element='prixmasse';
	public $table_element='prixmasse';
	public $fk_element='fk_prixmasse';
	public $picto = 'generic';
	public $ismultientitymanaged = 1;	// 0=No test on entity, 1=Test with field entity, 2=Test with link by societe

	/**
	 * {@inheritdoc}
	 */
	protected $table_ref_field = 'id';


	/**
     * Version id
     * @var string
     */
	public $fk_version = 0;

	/**
     * Object id
     * @var string
     */
	public $fk_object = 0;

	/**
     * Percent of price
     * @var string
     */
	public $percent = 0;

	/**
     * Display percent of price
     * @var string
     */
	public $percent_display = 0;

	/**
     * Id of user who created modification
     * @var string
     */
	public $fk_user_author = 0;

	/**
     * Product set ref
     * @var string
     */
	public $id = 0;

	/**
     * Propal HT before price mod
     * @var string
     */
	public $total_ht_before = 0;

	/**
     * Propal HT after price mod
     * @var string
     */
	public $total_ht_after = 0;

	

	/**
     * Product set ref
     * @var string
     */
	public $mods = array();

	/**
	 *  Constructor
	 *
	 *  @param      DoliDB		$db      Database handler
	 */
	function __construct($db)
	{
		global $langs;

		$this->db = $db;
	}

	/**
	 *    Check that ref and label are ok
	 *
	 *    @return     int         >1 if OK, <=0 if KO
	 */
	function check()
	{
		$err = 0;

		if ($this->fk_object < 0)
		$err++;

		if ($err > 0)
		{
			return 0;
		}
		else
		{
			return 1;
		}
	}

    /**
     *    Check properties of product are ok (like name, barcode, ...).
     *    All properties must be already loaded on object (this->barcode, this->barcode_type_code, ...).
     *
     *    @return     int		0 if OK, <0 if KO
     */
    function verify()
    {
        $this->errors=array();

        $result = 0;

        return $result;
    }

	/**
	 *	Insert product into database
	 *
	 *	@param	User	$user     		User making insert
	 *  @param	int		$notrigger		Disable triggers
	 *	@return int			     		Id of product/service if OK, < 0 if KO
	 */
	function create($user,$notrigger=0)
	{
		global $conf, $langs;

        $error=0;

		dol_syslog(get_class($this)."::create", LOG_DEBUG);

		$now=dol_now();

		$this->db->begin();

		// Check more parameters
		// If error, this->errors[] is filled
		$result = $this->verify();

		if ($result >= 0)
		{

			$this->fk_user_author = $user->id;

			// Produit non deja existant
			$sql = "INSERT INTO ".MAIN_DB_PREFIX."prixmasse (";
			$sql.= "datec";
			$sql.= ", tms";
			$sql.= ", entity";
			$sql.= ", fk_object";
			$sql.= ", fk_version";
			$sql.= ", fk_user_author";
			$sql.= ", total_ht_before";
			$sql.= ", total_ht_after";
			$sql.= ", percent";
			$sql.= ") VALUES (";
			$sql.= "'".$this->db->idate($now)."'";
			$sql.= ", '".$this->db->idate($now)."'";
			$sql.= ", ".$conf->entity;
			$sql.= ", ".$this->fk_object;
			$sql.= ", ".$this->fk_version;
			$sql.= ", ".$this->fk_user_author;
			$sql.= ", ".$this->total_ht_before;
			$sql.= ", ".$this->total_ht_after;
			$sql.= ", ".$this->percent;
			$sql.= ")";

			dol_syslog(get_class($this)."::Create", LOG_DEBUG);
			$result = $this->db->query($sql);
			if ( $result )
			{
				$id = $this->db->last_insert_id(MAIN_DB_PREFIX."prixmasse");

				if ($id > 0)
				{
					$this->id				= $id;

				}
				else
				{
					$error++;
					$this->error='ErrorFailedToGetInsertedId';
				}
			}
			else
			{
				$error++;
				$this->error=$this->db->lasterror();
			}


			if (! $error)
			{
				$this->db->commit();
				return $this->id;
			}
			else
			{
				$this->db->rollback();
				return -$error;
			}
        }
        else
       {
            $this->db->rollback();
            dol_syslog(get_class($this)."::Create fails verify ".join(',',$this->errors), LOG_WARNING);
            return -3;
        }

	}

	/**
	 *  Delete a product set from database (if not used)
	 *
	 *	@param      User	$user       Product id (usage of this is deprecated, delete should be called without parameters on a fetched object)
	 *  @param      int     $notrigger  Do not execute trigger
	 * 	@return		int					< 0 if KO, 0 = Not possible, > 0 if OK
	 */
	function delete(User $user, $notrigger=0)
	{
		// Deprecation warning
		if ($id > 0) {
			dol_syslog(__METHOD__ . " with parameter is deprecated", LOG_WARNING);
		}

		global $conf, $langs;

		$error=0;

		// Clean parameters
		if (empty($id)) $id=$this->id;
		else $this->fetch($id);

		// Check parameters
		if (empty($id))
		{
			$this->error = "Object must be fetched before calling delete";
			return -1;
		}
		
		$this->db->begin();


		// Delete product
		if (! $error)
		{
			$sqlz = "DELETE FROM ".MAIN_DB_PREFIX."prixmasse";
			$sqlz.= " WHERE rowid = ".$id;
			dol_syslog(get_class($this).'::delete', LOG_DEBUG);
			$resultz = $this->db->query($sqlz);

			if ( ! $resultz )
			{
				$error++;
				$this->errors[] = $this->db->lasterror();
			}

		}

		

		if (! $error)
		{
			$this->db->commit();
			return 1;
		}
		else
		{
			foreach($this->errors as $errmsg)
			{
				dol_syslog(get_class($this)."::delete ".$errmsg, LOG_ERR);
				$this->error.=($this->error?', '.$errmsg:$errmsg);
			}
			$this->db->rollback();
			return -$error;
		}

	}

	/**
	 *  Load a product set in memory from database
	 *
	 *  @param	int		$id      			Id of product/service to load
	 *  @param  string	$ref     			Ref of product/service to load
	 *  @return int     					<0 if KO, 0 if not found, >0 if OK
	 */
	function fetch($id = '', $fk_object='')
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::fetch fk_object=".$fk_object);

		// Check parameters
		if (!$id && ! $fk_object)
		{
			$this->error='ErrorWrongParameters';
			dol_print_error(get_class($this)."::fetch ".$this->error);
			return -1;
		}



		/*if ($fk_version == 0 && !empty($fk_object))
		{
			$prixmasseversion = new PrixMasseVersion($this->db);
			$result = $prixmasseversion->fetch(0, $fk_object);

			if ($result > 0)
			{
				$fk_version = $prixmasseversion->fk_version;
			}
		}*/

		$sql = "SELECT rowid, percent, fk_version, fk_object, fk_user_author, total_ht_before, total_ht_after, datec, tms";
		$sql.= " FROM ".MAIN_DB_PREFIX."prixmasse";
		$sql.= " WHERE entity IN (".getEntity($this->element).")";
		$sql.= !empty($id) ? " AND rowid = ".(int)$id : "";
		$sql.= !empty($fk_object) ? " AND fk_version = 0 AND fk_object = ".(int)$fk_object : "";
		$sql.= " ORDER BY rowid DESC";

		$resql = $this->db->query($sql);
		if ( $resql )
		{
			if ($this->db->num_rows($resql) > 0)
			{
				if ($id)
				{
					if ($obj = $this->db->fetch_object($resql))
					{
						$userstatic = new User($this->db);
						$userstatic->fetch($obj->fk_user_author);

						$this->id					= $obj->rowid;
						$this->percent				= $obj->percent;
						
						$this->total_ht_before		= $obj->total_ht_before;
						$this->total_ht_after		= $obj->total_ht_after;
						$this->percent_display		= $this->total_ht_before > 0 ? 100*($this->total_ht_after - $this->total_ht_before)/$this->total_ht_before : $this->percent;
						$this->fk_object			= $obj->fk_object;
						$this->fk_version			= $obj->fk_version;
						$this->fk_user_author		= $obj->fk_user_author;
						$this->fk_user				= $userstatic->getNomUrl();

						$this->date_creation			= $this->db->jdate($obj->datec);
						$this->date_modification		= $this->db->jdate($obj->tms);
					}
				}
				else
				{
					$this->mods = array();

					while ($obj = $this->db->fetch_object($resql))
					{
						$userstatic = new User($this->db);
						$userstatic->fetch($obj->fk_user_author);

						$mod = new stdClass();

						$mod->id					= $obj->rowid;
						$mod->percent				= $obj->percent;
						$mod->percent_display		= $obj->percent_display;
						$mod->fk_object				= $obj->fk_object;
						$mod->fk_user_author		= $obj->fk_user_author;
						$mod->total_ht_before		= $obj->total_ht_before;
						$mod->total_ht_after		= $obj->total_ht_after;
						$mod->percent_display		= $mod->total_ht_before > 0 ? 100*($mod->total_ht_after - $mod->total_ht_before)/$mod->total_ht_before : $mod->percent;

						$mod->fk_user				= $userstatic->getNomUrl();
						$mod->fk_version			= $obj->fk_version;

						$mod->date_creation			= $this->db->jdate($obj->datec);
						$mod->date_modification		= $this->db->jdate($obj->tms);

						$this->mods[] = $mod;
					}					
				}


				$this->db->free($resql);

				return 1;
			}
			else
			{
				return 0;
			}
		}
		else
		{
			dol_print_error($this->db);
			return -1;
		}
	}

	function setFromVersion($fk_object = 0, $fk_version = 0)
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::setFromVersion fk_object=".$fk_object);

		// Check parameters
		if (!$fk_object && ! $fk_version)
		{
			$this->error='ErrorWrongParameters';
			dol_print_error(get_class($this)."::setFromVersion ".$this->error);
			return -1;
		}

		$sql = " INSERT INTO ".MAIN_DB_PREFIX."prixmasse (";
		$sql.= " datec, tms, entity, fk_object, fk_version, fk_user_author, total_ht_before, total_ht_after, percent";
		$sql.= " ) SELECT datec, tms, entity, fk_object, 0, fk_user_author, total_ht_before, total_ht_after, percent";
		$sql.= " FROM ".MAIN_DB_PREFIX."prixmasse";
		$sql.= " WHERE fk_object = ".(int)$fk_object." AND fk_version = ".(int)$fk_version;
		
		$result = $this->db->query($sql);

		if ($result > 0)
		{
			return 1;
		}
		else
		{
			
			$this->error = $this->db->lasterror();
			return -1;
		}

	}

	function loadFromVersion($fk_object = 0, $fk_version = 0)
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::loadFromVersion fk_object=".$fk_object);

		// Check parameters
		if (!$fk_object && ! $fk_version)
		{
			$this->error='ErrorWrongParameters';
			dol_print_error(get_class($this)."::loadFromVersion ".$this->error);
			return -1;
		}
		
		$result = $this->deleteFromVersion($fk_object, 0);

		if ($result > 0)
		{
			return $this->setFromVersion($fk_object, $fk_version);
		}
		else
		{
			
			$this->error = $this->db->lasterror();
			return -1;
		}

	}

	function deleteFromVersion($fk_object = 0, $fk_version = 0)
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::deleteFromVersion fk_object=".$fk_object);

		// Check parameters
		if (!$fk_object && ! $fk_version)
		{
			$this->error='ErrorWrongParameters';
			dol_print_error(get_class($this)."::deleteFromVersion ".$this->error);
			return -1;
		}

		$sql = " DELETE FROM ".MAIN_DB_PREFIX."prixmasse";
		$sql.= " WHERE fk_version = ".(int)$fk_version." AND fk_object = ".(int)$fk_object;
		
		$result = $this->db->query($sql);

		if ($result > 0)
		{
			return 1;
		}
		else
		{
			
			$this->error = $this->db->lasterror();
			return -1;
		}

	}

	function copyCurrentVersion($fk_object = 0, $fk_version = 0)
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::setCurrentVersion fk_object=".$fk_object);

		// Check parameters
		if (!$fk_object && ! $fk_version)
		{
			$this->error='ErrorWrongParameters';
			dol_print_error(get_class($this)."::setCurrentVersion ".$this->error);
			return -1;
		}

		$sql = " UPDATE ".MAIN_DB_PREFIX."prixmasse SET fk_version = ".(int)$fk_version;
		$sql.= " WHERE fk_object = ".(int)$fk_object;
		
		$result = $this->db->query($sql);

		if ($result > 0)
		{
			return $this->setFromVersion($fk_object, $fk_version);
		}
		else
		{
			
			$this->error = $this->db->lasterror();
			return -1;
		}

	}

	/**
	 *	Return label of status of object
	 *
	 *	@param      int	$mode       0=long label, 1=short label, 2=Picto + short label, 3=Picto, 4=Picto + long label, 5=Short label + Picto
	 *	@return     string      	Label of status
	 */
	function getLibStatut($mode=0)
	{
		return $this->LibStatut($mode);

	}

	/**
	 *	Return label of a given status
	 *
	 *	@param      int		$status     Statut
	 *	@param      int		$mode       0=long label, 1=short label, 2=Picto + short label, 3=Picto, 4=Picto + long label, 5=Short label + Picto
	 *	@return     string      		Label of status
	 */
	function LibStatut($mode=0)
	{
		global $conf, $langs;

		$langs->load('prixmasse@prixmasse');

		return '';
	}


	/**
	 *	Apply mass modification on obct
	 *
	 *  @param	object	$object      			Propal object
	 *  @param  double	$mass_factor     		Mass factor
	 *  @return int     					<0 if KO, >0 if OK
	 */
	function applyMassMod(&$object, $mass_factor = 1)
	{
		global $conf, $langs;

		$prixMasseProduit = new PrixMasseProduit($this->db);
		$prixMasseProduit->fetch();

		$produits = $prixMasseProduit->produits;

		$error = 0;

		$lines = $object->lines;

		for ($i = 0; $i < sizeof($lines); $i++)
		{
			$line = $lines[$i];

			if ($line->fk_product > 0 && in_array($line->fk_product, array_keys($produits)))
			{
				continue;
			}

			$result = $object->updateline(
				$line->id, 
				$line->subprice * $mass_factor, 
				$line->qty, 
				$line->remise_percent, 
				$line->tva_tx, 
				$line->localtax1_tx, 
				$line->localtax2_tx, 
				$line->desc, 
				'HT', 
				$line->info_bits, 
				$line->special_code, 
				$line->fk_parent_line, 
				0, 
				$line->fk_fournprice, 
				$line->pa_ht, 
				$line->label, 
				$line->product_type,
				$line->date_start, 
				$line->date_end, 
				$line->array_options, 
				$line->fk_unit, 
				$line->multicurrency_subprice);

			if ($result < 0)
			{
				$error++;
			}
		}

		$object->fetch($object->id);
		
		return $error > 0 ? -$error : 1;
	}

}
