<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/prixmasse/class/prixmasse.class.php
 *  \ingroup    prixmasse
 *  \brief      File of class to manage predefined products sets
 */
require_once DOL_DOCUMENT_ROOT.'/core/class/commonobject.class.php';
require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';

/**
 * Class to manage products or services
 */
class PrixMasseProduit extends CommonObject
{
	public $element='prixmasse_produits';
	public $table_element='prixmasse_produits';
	public $fk_element='fk_prixmasse_produit';
	public $picto = 'generic';
	public $ismultientitymanaged = 1;	// 0=No test on entity, 1=Test with field entity, 2=Test with link by societe

	/**
	 * {@inheritdoc}
	 */
	protected $table_ref_field = 'id';


	/**
     * Product id
     * @var string
     */
	public $fk_produit = 0;

	/**
     * Id
     * @var string
     */
	public $id = 0;	

	/**
     * Product set ref
     * @var string
     */
	public $produits = array();

	/**
	 *  Constructor
	 *
	 *  @param      DoliDB		$db      Database handler
	 */
	function __construct($db)
	{
		global $langs;

		$this->db = $db;
	}

	/**
	 *	Insert product into database
	 *
	 *	@param	User	$user     		User making insert
	 *  @param	int		$notrigger		Disable triggers
	 *	@return int			     		Id of product/service if OK, < 0 if KO
	 */
	function create($user,$notrigger=0)
	{
		global $conf, $langs;

        $error=0;

		dol_syslog(get_class($this)."::create", LOG_DEBUG);

		$now=dol_now();

		$this->db->begin();

        // Produit non deja existant
        $sql = "INSERT INTO ".MAIN_DB_PREFIX."prixmasse_produits (";
		$sql.= "datec";
		$sql.= ", tms";
        $sql.= ", entity";
        $sql.= ", fk_produit";
        $sql.= ") VALUES (";
		$sql.= "'".$this->db->idate($now)."'";
		$sql.= ", '".$this->db->idate($now)."'";
        $sql.= ", ".$conf->entity;
        $sql.= ", ".$this->fk_produit;
        $sql.= ")";

        dol_syslog(get_class($this)."::Create", LOG_DEBUG);
        $result = $this->db->query($sql);
        if ( $result )
        {
            $id = $this->db->last_insert_id(MAIN_DB_PREFIX."prixmasse_produits");

            if ($id > 0)
            {
                $this->id				= $id;

            }
            else
            {
                $error++;
                $this->error='ErrorFailedToGetInsertedId';
            }
        }
        else
        {
            $error++;
            $this->error=$this->db->lasterror();
        }


        if (! $error)
        {
            $this->db->commit();
            return $this->id;
        }
        else
        {
            $this->db->rollback();
            return -$error;
        }
	}

	/**
	 *  Delete a product set from database (if not used)
	 *
	 *	@param      User	$user       Product id (usage of this is deprecated, delete should be called without parameters on a fetched object)
	 *  @param      int     $notrigger  Do not execute trigger
	 * 	@return		int					< 0 if KO, 0 = Not possible, > 0 if OK
	 */
	function delete(User $user, $notrigger=0)
	{
		// Deprecation warning
		if ($id > 0) {
			dol_syslog(__METHOD__ . " with parameter is deprecated", LOG_WARNING);
		}

		global $conf, $langs;

		$error=0;

		// Clean parameters
		if (empty($id)) $id=$this->id;
		else $this->fetch($id);

		// Check parameters
		if (empty($id))
		{
			$this->error = "Object must be fetched before calling delete";
			return -1;
		}
		
		$this->db->begin();


		// Delete product
		if (! $error)
		{
			$sqlz = "DELETE FROM ".MAIN_DB_PREFIX."prixmasse_produits";
			$sqlz.= " WHERE rowid = ".$id;
			dol_syslog(get_class($this).'::delete', LOG_DEBUG);
			$resultz = $this->db->query($sqlz);

			if ( ! $resultz )
			{
				$error++;
				$this->errors[] = $this->db->lasterror();
			}

		}

		

		if (! $error)
		{
			$this->db->commit();
			return 1;
		}
		else
		{
			foreach($this->errors as $errmsg)
			{
				dol_syslog(get_class($this)."::delete ".$errmsg, LOG_ERR);
				$this->error.=($this->error?', '.$errmsg:$errmsg);
			}
			$this->db->rollback();
			return -$error;
		}

	}

	/**
	 *  Load a product set in memory from database
	 *
	 *  @param	int		$id      			Id of product/service to load
	 *  @param  string	$ref     			Ref of product/service to load
	 *  @return int     					<0 if KO, 0 if not found, >0 if OK
	 */
	function fetch($id = '', $fk_produit = '')
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::fetch fk_produit=".$fk_produit);
        
		$sql = "SELECT rowid, fk_produit, datec, tms";
		$sql.= " FROM ".MAIN_DB_PREFIX."prixmasse_produits";
		$sql.= " WHERE entity IN (".getEntity($this->element).")";
        $sql.= !empty($id) ? " AND rowid = ".(int)$id : "";
        $sql.= !empty($fk_produit) ? " AND fk_produit = ".(int)$fk_produit : "";
		$sql.= " ORDER BY rowid DESC";

		$resql = $this->db->query($sql);
		if ( $resql )
		{
			if ($this->db->num_rows($resql) > 0)
			{
				if (!empty($id) || !empty($fk_produit))
				{
					if ($obj = $this->db->fetch_object($resql))
					{
						$this->id					= $obj->rowid;
						$this->fk_produit			= $obj->fk_produit;

						$this->date_creation			= $this->db->jdate($obj->datec);
						$this->date_modification		= $this->db->jdate($obj->tms);
					}
				}
				else
				{
					$this->produits = array();

					while ($obj = $this->db->fetch_object($resql))
					{
                        $produit = new Product($this->db);
                        $produit->fetch($obj->fk_produit);

						$this->produits[$obj->fk_produit] = $produit;
					}					
				}


				$this->db->free($resql);

				return 1;
			}
			else
			{
				return 0;
			}
		}
		else
		{
			dol_print_error($this->db);
			return -1;
		}
	}
}
